@@ -52,6 +52,7 @@ import Data.Char(chr,ord)
 import Data.Monoid(Monoid(..))
 import Data.ByteString(ByteString)
 import qualified Data.ByteString as B
+import qualified Data.ByteString.Char8 as B8
 import qualified Data.ByteString.Internal as B
 import Data.ByteString.Unsafe
 import System.IO(Handle)
@@ -122,13 +123,13 @@ putUTF8 :: (UTF8 a) => a -> IO ()
 putUTF8 = flip withUTF8 B.putStr
 
 putUTF8Ln :: (UTF8 a) => a -> IO ()
-putUTF8Ln = flip withUTF8 B.putStrLn
+putUTF8Ln = flip withUTF8 B8.putStrLn
 
 hPutUTF8 :: (UTF8 a) => Handle -> a -> IO ()
 hPutUTF8 h = flip withUTF8 (B.hPut h)
 
 hPutUTF8Ln :: (UTF8 a) => Handle -> a -> IO ()
-hPutUTF8Ln h = flip withUTF8 (B.hPutStrLn h)
+hPutUTF8Ln h = flip withUTF8 (B8.hPutStrLn h)
 
 readUTF8File :: (UTF8 a) => FilePath -> IO a
 readUTF8File = (return . decode =<<) . B.readFile
@@ -251,15 +252,27 @@ encodeUTF8' [] = []
 -- can use Word# literalls
 -- ==> 0xff00ff00##
 encodeUTF8' ((W32# w):xs)
+#if MIN_VERSION_base(4,7,0)
+  | I# (w`ltWord#`(int2Word# 0x80#)) /= 0 =
+#else
   | w`ltWord#`(int2Word# 0x80#) =
+#endif
       [W8# w] : encodeUTF8' xs
+#if MIN_VERSION_base(4,7,0)
+  | I# (w`ltWord#`(int2Word# 0x800#)) /= 0 =
+#else
   | w`ltWord#`(int2Word# 0x800#) =
+#endif
       [ W8#(w`uncheckedShiftRL#`6#
               `or#`(int2Word# 0xc0#))
       , W8#(w`and#`(int2Word# 0x3f#)
               `or#`(int2Word# 0x80#))
       ] : encodeUTF8' xs
+#if MIN_VERSION_base(4,7,0)
+  | I# (w`ltWord#`(int2Word# 0xf0000#)) /= 0 =
+#else
   | w`ltWord#`(int2Word# 0xf0000#) =
+#endif
       [ W8#(w`uncheckedShiftRL#`12#
               `or#`(int2Word# 0xe0#))
       , W8#(w`uncheckedShiftRL#`6#
@@ -268,7 +281,11 @@ encodeUTF8' ((W32# w):xs)
       , W8#(w`and#`(int2Word# 0x3f#)
               `or#`(int2Word# 0x80#))
       ] : encodeUTF8' xs
+#if MIN_VERSION_base(4,7,0)
+  | I# (w`ltWord#`(int2Word# 0xe00000#)) /= 0 =
+#else
   | w`ltWord#`(int2Word# 0xe00000#) =
+#endif
       [ W8#(w`uncheckedShiftRL#`18#
               `or#`(int2Word# 0xf0#))
       , W8#(w`uncheckedShiftRL#`12#
